/** Represents a scope within a TableOfVars instance */
function TableOfVarsInstancelike(
    parentTable,    // TableOfVarsAbstract instance containing this instance
    pointer         // Optional: Preferred memory address
) {
    TableOfVars.parent.call(this, parentTable);
    this.pointer = ((Number.isFinite(pointer) && parentTable.canAllocate(pointer, this.size)) || (pointer == -Infinity)) ? pointer : parentTable.allocateMemory(this.size);
    this.nodeData = [ parentTable.registerInstance(this, this.baseType, this.dimensions) ];
}
TableOfVarsInstancelike.parent = TableOfVarsAbstract;
TableOfVarsInstancelike.prototype = Object.create(TableOfVarsInstancelike.parent.prototype);
TableOfVarsInstancelike.prototype.constructor = TableOfVarsInstancelike;

/** Memory location of instance */
TableOfVarsInstancelike.prototype.pointer = undefined;

/** Size of needed space in bytes */
TableOfVarsInstancelike.prototype.size = 1;

/** Internal name of (base) type */
TableOfVarsInstancelike.prototype.baseType = null;

/** Number of dimensions */
TableOfVarsInstancelike.prototype.dimensions = 0;

/**
* Create name cells's DOM node
* @return Node      DOM node beeing name cell's content
*/
TableOfVarsInstancelike.prototype.createCellContentName = function(
    doc,        // Document instance
    name        // Variable's name to display or DOM node
) {
    const node = TableOfVarsInstancelike.parent.prototype.createCellContentName.call(this, doc, name);
    ScoTools.createHtml('span', node, { 'class': 'nested' }, '↳', node.firstChild);
    return node;
}

/**
* Create edit cells's DOM node
* @return Node      DOM node beeing edit cell's content
*/
TableOfVarsInstancelike.prototype.createCellContentEdit = function(
    doc,            // Document instance
    removeCallback  // Optional: Function to call when remove button is pressed
) {
    return null;
}

/**
* Start a new scope referencing this instance
* @return undefined
*/
TableOfVarsInstancelike.prototype.startScope = function() {
    this.addScope(this.pointer);
}

/** Description of this type in error messages */
TableOfVarsInstancelike.prototype.descType = 'Instanz (allgemein)';

/**
* Removes this instance if possible
* @return undefined
*/
TableOfVarsInstancelike.prototype.tryRemove = function() {
    const problems = this.getExternalReferences(this.pointer);
    if (!problems.length) {
        this.remove();
        return;
    }
    switch (problems[0].type) {
        case 'scope':
            this._addfailed('Solange es der Wert von „this“ in einem Sichtbarkeitsbereich ist, kann das ' + this.descType + ' „' + this.pointerToAddress(this.pointer) + '“ nicht entfernt werden');
            break;
        case 'var':
            this._addfailed('Da eine Variable namens „' + problems[0].obj.varName + '“ drauf zeigt, kann das ' + this.descType + ' „' + this.pointerToAddress(this.pointer) + '“ nicht entfernt werden');
            break;
        default:
            this._addfailed(this.descType + ' „' + this.pointerToAddress(this.pointer) + '“ kann nicht entfernt werden');
            break;
    }
}

/**
* Force remove this instance
* @return undefined
*/
TableOfVarsInstancelike.prototype.remove = function() {
    this.removeInstance(this, this.nodeData);
}
